@@ -1,6 +1,8 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at https://mozilla.org/MPL/2.0/.
@@ -9,8 +11,7 @@
  * information regarding copyright ownership.
  */
 
-#ifndef ISC_LANG_H
-#define ISC_LANG_H 1
+#pragma once
 
 /*! \file isc/lang.h */
 
@@ -21,5 +22,3 @@
 #define ISC_LANG_BEGINDECLS
 #define ISC_LANG_ENDDECLS
 #endif /* ifdef __cplusplus */
-
-#endif /* ISC_LANG_H */
