@@ -13,18 +13,17 @@
 
 #pragma once
 
+#include <isc/loop.h>
 #include <isc/netmgr.h>
 #include <isc/result.h>
-#include <isc/task.h>
 #include <isc/timer.h>
 
 typedef struct isc_managers isc_managers_t;
 
-isc_result_t
-isc_managers_create(isc_mem_t *mctx, size_t workers, size_t quantum,
-		    isc_nm_t **netmgrp, isc_taskmgr_t **taskmgrp,
-		    isc_timermgr_t **timermgrp);
+void
+isc_managers_create(isc_mem_t **mctx, uint32_t workers,
+		    isc_loopmgr_t **loopmgrp, isc_nm_t **netmgrp);
 
 void
-isc_managers_destroy(isc_nm_t **netmgrp, isc_taskmgr_t **taskmgrp,
-		     isc_timermgr_t **timermgrp);
+isc_managers_destroy(isc_mem_t **mctx, isc_loopmgr_t **loopmgrp,
+		     isc_nm_t **netmgrp);
